% Load housing data in CSV into a matlab Table

T = readtable('housing_regression.xlsx');           % Read file
T.ocean_proximity=categorical(T.ocean_proximity);   % Set variable as categorical

% Choose 5000 rows of data randomly
rng(100) 
idx=randperm(size(T,1));
T1 = T(idx(1:5000),:); 

%Train Raw data
rng(201);
[trainInd,~,testInd] = dividerand(size(T1,1),0.7,0,0.3);

Ttest  = T1(testInd',:);
Ttrain = T1(trainInd',:);

% Launch Regression Learner App to fit regression models to data
%regressionLearner(Ttrain,'median_house_value');
%%
clc;
workspace;

fprintf(1,['End of Demo 1  - Using Regression Learner App\n'])


%% Step 1: Visualize the data, Summarize variables, data cleaning, pre-processing if needed
clc;
head(T,10)                                          % show first 10 rows of data table
format compact 
summary(T)                                          % Show the summary of data variables

% From the summary, variable total_bedrooms has 207 missing values.
% Impute missing values with median
jj= isnan(table2array(T(:,5)));
s = summary(T(:,5));
rpl = s.total_bedrooms.Median.*jj;
rpl(rpl==0)= table2array(T(~jj,5));
T(:,5)=table(rpl);

%Only 5 entries with ISLAND. Remove these from data set
jj= find(T.ocean_proximity=='ISLAND');
T(jj,:)=[];

% Reduce data size to 5000 observations to run quickly and demonstrate key
% points in modeling
rng(100) %For reproducibility
idx=randperm(size(T,1));
T1 = T(idx(1:5000),:); 

% Divide the data set into training, validation and TestIndx
rng(201);
[trainInd,~,testInd] = dividerand(size(T1,1),0.7,0,0.3);
Ttest  = T1(testInd',:);
Ttrain = T1(trainInd',:);

%% Visualize the data
nVar =size(T1,2);

figure
for ii =1:nVar
subplot(4,3,ii)
xx=table2array(T1(:,ii));
histogram(xx)
b = regexprep(T1.Properties.VariableNames{ii},'_',' ','emptymatch');
title(b)
end
%%  Convert categorical variable to nummerical 
op_vbl= size(T1.ocean_proximity);
op_vbl(T1.ocean_proximity =='<1H OCEAN')=1;
op_vbl(T1.ocean_proximity =='INLAND')=2;
op_vbl(T1.ocean_proximity=='NEAR BAY')=3;
op_vbl(T1.ocean_proximity =='NEAR OCEAN')=4;
op_vbl=op_vbl(:);
T1= [T1 table(op_vbl)];

T2 = movevars(T1,'op_vbl','Before','median_house_value');
T1 = T2; % T1 now has a numeric dummy variable for the categorical variable
clear T2;


%% Find variable correlations to each other and the response Median house value

select_vars =[3 4 8 10 11];
[R,pp] = corr(table2array(T1(:,select_vars)));

kk=0;
figure
for ii = 1:length(select_vars)
    b1 = regexprep(T1.Properties.VariableNames{select_vars(ii)},'_',' ','emptymatch');
    for jj=1:length(select_vars)
        kk=kk+1;
    subplot(5,5,kk)
    xx=table2array(T1(:,[select_vars(ii) select_vars(jj)]));
    plot(xx(:,1),xx(:,2),'.')
    b2 = regexprep(T1.Properties.VariableNames{select_vars(jj)},'_',' ','emptymatch');
    xlabel(b1); ylabel(b2)
    if pp(ii,jj)<0.05
        title(['r = ',num2str(R(ii,jj),2)],'fontsize',12)
    end
    end
end

%% Split into Training and Test Data and fit linear regression model
clc; 
Ttest  = T1(testInd',[1:9 11]);
Ttrain = T1(trainInd',[1:9 11]);

% Linear Regression model
mdl = fitlm(Ttrain);
fcn = @(Xtr, Ytr, Xte) predict( fitlm(Xtr,Ytr), Xte);
% perform cross-validation, and return average MSE across folds
rmse_val = crossval('mse', table2array(T1(trainInd',[1:8 10])), table2array(T1(trainInd',11)),'Predfun',fcn, 'kfold',10);
rmse_val=sqrt(rmse_val);

pred_mdl = predict(mdl,Ttest);
rmse_test = sqrt(mean((pred_mdl-table2array(Ttest(:,10))).^2,'omitnan'));

clc;
fprintf(1,'Linear regression Completed...\n\n')
fprintf(1,'Linear Regression (all 9 predictors)\n\t\t\t Validation RMSE = %6.3f\n\t\t\t Test RMSE = %6.3f\n',rmse_val,rmse_test)
fprintf(1,'\n\n\n\n\n\n\n\n\n')
pause(2)
mdl

%%
clc;

% Remove the insignificant levels of ocean_proximity. Then create the new  model
temp_op = dummyvar(T1.ocean_proximity);
newTtrain = [T1(trainInd',1:8) array2table(temp_op(trainInd',2),'VariableNames',{'op_inland'}) T1(trainInd',11)];
newTtest  = [T1(testInd',1:8) array2table(temp_op(testInd',2),'VariableNames',{'op_inland'}) T1(testInd',11)];

mdl2 = fitlm(newTtrain);
% perform cross-validation, and return average MSE across folds
rmse_val2 = crossval('mse', table2array(newTtrain(:,1:end-1)), table2array(newTtrain(:,end)),'Predfun',fcn, 'kfold',10);
rmse_val2=sqrt(rmse_val2);

pred_mdl2 = predict(mdl2,newTtest);
rmse_test2 = sqrt(mean(((pred_mdl2-table2array(newTtest(:,10))).^2),'omitnan'));

fprintf(1,'Running linear regression with fewer features ...\n\n')
fprintf(1,'Completed...\n\n')
fprintf(1,'Linear Regression (reduced set of predictors)\n\t\t\t Validation RMSE = %6.3f\n\t\t\t Test RMSE = %6.3f\n',rmse_val2,rmse_test2)
fprintf(1,'Compare to Linear Regression with all 9 predictors)\n\t\t\t Validation RMSE = %6.3f\n\t\t\t Test RMSE = %6.3f\n',rmse_val,rmse_test)
pause(5)
fprintf(1,'\n\n\n\n\n\n\n')
mdl2

%% SVM model with fewer variables
clc;
fprintf(1,'Running SVM model with linear kernel....');


%Linear Kernel - standardized variables, 10 fold cross-validation
rng default  % For reproducibility
mdl_svm = fitrsvm(newTtrain,'median_house_value','standardize',true,'KernelFunction','linear','KFold',5,'CategoricalPredictors','op_inland');
lStd = kfoldLoss(mdl_svm); % Validation error
mdl_svm1 = fitrsvm(newTtrain,'median_house_value','standardize',true,'KernelFunction','linear','CategoricalPredictors','op_inland');
% Apply model to test data
pred1 = predict(mdl_svm1,newTtest);
rmse_test3 = sqrt(mean(((pred1-table2array(newTtest(:,end))).^2),'omitnan')); % Test RMSE

%print results

fprintf(1,'Completed.....\n\n')
fprintf(1,'SVM: Linear Kernel\n\t\t\t Validation RMSE = %6.3f\n\t\t\t Test RMSE = %6.3f\n',sqrt(lStd),rmse_test3)
fprintf(1,'\n\n\n\n')
fprintf(1,'Running SVM model with Gaussian kernel....');

%Gaussian Kernel - standardized, holdout CV
rng default  % For reproducibility
mdl2_svm = fitrsvm(newTtrain,'median_house_value','KernelFunction','gaussian','KernelScale','auto',...
    'Standardize',true,'KFold',5,'CategoricalPredictors','op_inland');
%mdl2_svm.ConvergenceInfo.Converged
lStd2 =kfoldLoss(mdl2_svm); %MSE of CV

mdl2_svm2 = fitrsvm(newTtrain,'median_house_value','KernelFunction','gaussian','KernelScale','auto',...
    'Standardize',true,'CategoricalPredictors','op_inland');
% Apply model to test data
pred2 = predict(mdl2_svm2,newTtest);
rmse_test4 = sqrt(mean(((pred2-table2array(newTtest(:,end))).^2),'omitnan'));  % Test RMSE

%Print results
fprintf(1,'Completed.....\n\n')

fprintf(1,'SVM: Gaussian Kernel\n\t\t\t Validation RMSE = %6.3f\n\t\t\t Test RMSE = %6.3f\n',sqrt(lStd2),rmse_test4)


% plot outputs
[actual,ndx] = sort(table2array(newTtest(:,end)));
figure
hold on;
plot(actual,pred_mdl2(ndx),'b*')
plot(actual,pred1(ndx),'r*')
plot(actual,pred2(ndx),'ko');
legend('Lin Reg','SVM-lin','SVM-Gau');
xlabel('Actual Housing values','fontsize',12);
ylabel('Predicted values','fontsize',12)
title('Predicted house values: Regression models')

pause(2)
fprintf(1,'\n\n\nEnd of Demo 2 - Modeling using Linear Regression and SVM\n')

%% Lasso regression - to pick the most important predictors
clc;
fprintf(1,'Running lasso regression...')

rng('default')
X = table2array(T1(trainInd',[1:8 10])); y = table2array(T1(trainInd',end)); 
[B,FitInfo] = lasso(X,y,'CV',10,'PredictorNames',T1.Properties.VariableNames([1:8 10]));

idxLambda1SE = FitInfo.Index1SE;
sparseModelPredictors = FitInfo.PredictorNames(B(:,idxLambda1SE)~=0);
coef = B(:,idxLambda1SE);
coef0 = FitInfo.Intercept(idxLambda1SE);



figure
lassoPlot(B,FitInfo,'PlotType','CV');
legend('show') % Show legend

% predict using the lasso regularized coefficients
Xtest = table2array(T1(testInd',[1:8 10])); ytest = table2array(T1(testInd',end)); 
yhat = Xtest*coef + coef0;
rmse_lasso =sqrt(mean((yhat-ytest).^2,'omitnan'));

fprintf(1,'completed...\n');
fprintf(1,'\n\nChosen oefficient matrix is: \n');
coef
fprintf(1,'\nChosen predictors are: \n'),
fprintf(1,'%s\n',string(sparseModelPredictors))
fprintf(1,'\nEliminated predictors are: %s\t%s\n',string(FitInfo.PredictorNames(4)),'and  OP_Inland');
fprintf(1,'\n\nTest RMSE with 7 features = %6.3f\n',rmse_lasso);
